//
// Created by robin on 1/9/18.
//

#include "dns/QDnsLabel.h"
#include <QDataStream>

namespace QDns
{

    QString QDnsLabel::parse(const QByteArray& packetData, quint32 pos, quint8 maxRecursion)
    {
        QDataStream dataStream(packetData);
        dataStream.skipRawData(pos);
        return parse(packetData, dataStream, maxRecursion);
    }

    QString QDnsLabel::parse(const QByteArray& packetData, QDataStream& dataStream, quint8 maxRecursion)
    {
        if (maxRecursion == 0)
        {
            return QString::null;
        }

        QString fullLabel = "";

        if (dataStream.device()->bytesAvailable() < 1)
        {
            return QString::null;
        }

        quint8 labelSize;
        dataStream >> labelSize;
        while (labelSize != 0)
        {
            if (labelSize >> 6 == 3)
            {
                if (dataStream.device()->bytesAvailable() < 1)
                {
                    return QString::null;
                }
                quint8 labelSize2;
                dataStream >> labelSize2;
                quint16 pointer = ((labelSize << 8) | labelSize2) - (quint16) 49152;
                QDataStream dataStream2(packetData);
                dataStream2.skipRawData(pointer);
                auto labelEnd = parse(packetData, dataStream2, maxRecursion - (quint8) 1);
                if (labelEnd.isNull())
                {
                    return QString::null;
                }
                if (!fullLabel.isEmpty())
                {
                    fullLabel.append(".");
                }
                fullLabel.append(labelEnd);
                break;
            }
            else if (labelSize >> 6 == 0)
            {

                if (dataStream.device()->bytesAvailable() < labelSize + 1)
                {
                    return QString::null;
                }

                char label[labelSize];
                int read = dataStream.readRawData(label, labelSize);
                if (read != labelSize)
                {
                    return QString::null;
                }
                if (!fullLabel.isEmpty())
                {
                    fullLabel.append(".");
                }
                fullLabel.append(QByteArray(label, labelSize));
                dataStream >> labelSize;
            }
            else
            {
                return QString::null;
            }
        }
        return fullLabel;
    }

    QByteArray QDnsLabel::serialize(const QString& label)
    {
        QByteArray data;
        QDataStream dataStream(&data, QIODevice::WriteOnly);
        serialize(label, dataStream);
        return data;
    }

    void QDnsLabel::serialize(const QString& label, QDataStream& dataStream)
    {
        if (label.size() != 0)
        {
            for (const auto& part : label.split("."))
            {
                dataStream << (quint8) part.length();
                for (const auto& c : part)
                {
                    dataStream << (quint8) c.toLatin1();
                }
            }
        }
        dataStream << (quint8) 0;
    }

}