//
// Created by robin on 1/8/18.
//

#include <QDataStream>
#include <includes/dns/QDnsLabel.h>
#include "dns/QDnsQuery.h"

namespace QDns
{

    QDnsQuery::QDnsQuery()
        : m_isValid(true)
        , m_type(0)
        , m_class(0)
    {
    }

    QDnsQuery QDnsQuery::parse(const QByteArray& packetData, quint32 pos)
    {
        QDataStream dataStream(packetData);
        dataStream.skipRawData(pos);
        return parse(packetData, dataStream);
    }

    QDnsQuery QDnsQuery::parse(const QByteArray& packetData, QDataStream& dataStream)
    {
        QDnsQuery query;

        query.m_name = QDnsLabel::parse(packetData, dataStream);
        if (query.m_name.isNull() || dataStream.device()->bytesAvailable() < 4)
        {
            query.m_isValid = false;
        }
        else
        {
            dataStream >> query.m_type;
            dataStream >> query.m_class;
        }

        return query;
    }

    QByteArray QDnsQuery::serialize() const
    {
        QByteArray byteArray;
        QDataStream dataStream(&byteArray, QIODevice::WriteOnly);
        serialize(dataStream);
        return byteArray;
    }

    void QDnsQuery::serialize(QDataStream& dataStream) const
    {
        QDnsLabel::serialize(m_name, dataStream);
        dataStream << m_type;
        dataStream << m_class;
    }

    bool QDnsQuery::isValid() const
    {
        return m_isValid;
    }

    const QString& QDnsQuery::getName() const
    {
        return m_name;
    }

    void QDnsQuery::setName(const QString& name)
    {
        m_name = name;
    }

    quint16 QDnsQuery::getType() const
    {
        return m_type;
    }

    void QDnsQuery::setType(quint16 type)
    {
        m_type = type;
    }

    quint16 QDnsQuery::getClass() const
    {
        return m_class;
    }

    void QDnsQuery::setClass(quint16 aClass)
    {
        m_class = aClass;
    }
}